      SUBROUTINE J5INITZC(A,LDA,NB,NLB,IPROW,IPCOL,KBPIV,NMOD)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
* ARGUMENTS
*
      INTEGER LDA
      COMPLEX*16 A(LDA,*)
      INTEGER NB,NLB,IPROW,IPCOL,KBPIV,NMOD
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5INITZC IS INITIALIZE RESIDUAL BLOCK AND UPPER DIAGONAL ELEMENTS.
*
*  ARGUMENTS
*  =========
*
*  A       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          ON ENTRY,
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*          ON EXIT,
*          INITIALIZE RESIDUAL BLOCK AND UPPER DIAGONAL ELEMENTS.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  NMOD    (INPUT) INTEGER
*          THE NUMBER OF LAST BLOCK ELEMENTS.
*
* ----------------------------------------------------------------------
*
* LOCAL VALIABLES
*
      INTEGER I,J,L,IS,JS,JE
*
* PARAMETER
*
      COMPLEX*16 ZERO
      PARAMETER (ZERO=DCMPLX(0.0D0,0.0D0))
*
* INITIALIZE UPPER DIAGONAL ELEMENTS
*
      IF(IPROW .EQ. IPCOL)THEN
         DO L=1,NLB
            JS=(L-1)*NB+2
            JE=L*NB
            IS=(L-1)*NB+1
            DO J=JS,JE
               DO I=IS,J-1
                  A(I,J)=ZERO
               ENDDO
            ENDDO
         ENDDO
      END IF
      IF(IPCOL .GT. KBPIV)THEN
         DO J=(NLB-1)*NB+1,NLB*NB
            DO I=1,NLB*NB
               A(I,J)=ZERO
            END DO
         END DO
      END IF
*
* INITIALIZE RESIDUAL BLOCK
*
      IF(IPROW .GT. KBPIV)THEN
         DO J=1,NLB*NB
            DO I=(NLB-1)*NB+1,NLB*NB
               A(I,J)=ZERO
            END DO
         END DO
      END IF
*
      IF((IPCOL .EQ. KBPIV) .AND. (IPROW .LE. KBPIV))THEN
         DO J=(NLB-1)*NB+NMOD+1,NLB*NB
            DO I=1,NLB*NB
               A(I,J)=ZERO
            END DO
         END DO
      END IF
*
      IF((IPROW .EQ. KBPIV) .AND. (IPCOL .LE. KBPIV))THEN
         DO J=1,NLB*NB
            DO I=(NLB-1)*NB+NMOD+1,NLB*NB
               A(I,J)=ZERO
            END DO
         END DO
      END IF
*
      RETURN
      END
