      SUBROUTINE J5MAKVEC(N,A,LDA,V,TAU,JVBLK,NP,IP,NB,NMOD,
     *                    NPROW,NPCOL,IPROW,IPCOL,NLB,JVE,NGB,JVNUM,
     *                    UF,USEN,UREC,WK,
     *                    MPI_COMM_ROW,MPI_COMM_COL)
*
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      INTEGER N,LDA,NP,NB,JVNUM
      COMPLEX*16 A(LDA,*),V(N,*),TAU(N)
      COMPLEX*16 UF(NB,NPROW*NLB*NB),USEN(NB,NLB*NB),UREC(NB,NLB*NB)
      COMPLEX*16 WK(4,JVNUM)
      INTEGER JVBLK(NP+1)
      INTEGER NMOD,NPROW,NPCOL,IPROW,IPCOL,NLB,IP,JVE,NGB
      INTEGER MPI_COMM_ROW,MPI_COMM_COL
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5MAKVEC COMPUTE HOUSEHOLDER INVERSE TRANSFORMATION.
*
*  ARGUMENTS
*  =========
*
*  N       (INPUT) INTEGER
*          THE ORDER OF THE MATRIX.
*          N >= 2
*
*  A       (INPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          THE PART OF A ELEMENTARY REFLECTORS.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  V       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N,*)
*          ON ENTRY,
*          UNIT MATRIX.
*          ON EXIT,
*          RESULT OF HOUSEHOLDER INVERSE TRANSFORMATION.
*
*  JVBLK   (INPUT) INTEGER ARRAY, DIMENSION(NP+1).
*          GLOBAL ADDRESS LIST FOR EIGENVECTORS.
*
*  NP      (INPUT) INTEGER
*          TOTAL NUMBER OF MPI PROCESSES.
*
*  IP      (INPUT) INTEGER
*          NUMBER OF MPI PROCESSOR.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NMOD    (INPUT) INTEGER
*          THE NUMBER OF LAST BLOCK ELEMENTS.
*
*  NPROW   (INPUT) INTEGER
*          THE ROW SIZE OF PROCESS-MAP.
*
*  NPCOL   (INPUT) INTEGER
*          THE COLUMN SIZE OF PROCESS-MAP.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  JVE     (INPUT) INTEGER
*          END INDEX OF V.
*
*  NGB     (INPUT) INTEGER
*          TOTAL NUMBER OF BLOCKS.
*
*  JVNUM   (INPUT) INTEGER
*          THE NUMBER OF COLUMNS TO V.
*
*  UF      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB,NPROW*NLB*NB).
*
*  USEN    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB,NLB*NB).
*
*  UREC    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB,NLB*NB).
*
*  WK      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (4,JVNUM).
*
*  MPI_COMM(INPUT) INTEGER
*  _ROW    THE COMMUNICATOR OF ROW.
*
*  MPI_COMM(INPUT) INTEGER
*  _COL    THE COMMUNICATOR OF COLUMN.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 CS,CA
      COMPLEX*16 CS11,CS12,CS13,CS14
      COMPLEX*16 CS21,CS22,CS23,CS24
      COMPLEX*16 P11,P12,P13,P14
      COMPLEX*16 P21,P22,P23,P24
      COMPLEX*16 ZW12,ZW13,ZW14,ZW23,ZW24,ZW34
      COMPLEX*16 ZC12,ZC13,ZC14,ZC23,ZC24,ZC34
      INTEGER I,J,K,L,J1,IL,IG
      INTEGER KBPIV,IADDR,IS,JS,KGB,KTMP,KMOD
      INTEGER ISEN,IREC,ISIZE
      INTEGER IERR1,IERR2,ITAG1
      INTEGER REQ_S
      INTEGER SWK(MPI_STATUS_SIZE),RWK(MPI_STATUS_SIZE)
      INTEGER ISTART,IEND,IB,LB
      INTEGER NBMOD,JMOD
*
* PARAMETER
*
      COMPLEX*16 ZERO
      PARAMETER (ZERO=DCMPLX(0.0D0,0.0D0))
      INTEGER JAUNRL,JVUNRL,IBLK
      PARAMETER (IBLK=1000,JAUNRL=4,JVUNRL=2)
*
      KBPIV=MOD(NGB-1,NPCOL)+1
      IADDR=(NLB-1)*NB
      IS=(NGB-1)*NB
      ISIZE=NB*NB
      KMOD=0
      IF(NMOD .EQ. 1)THEN
         KMOD=1
      END IF
      NBMOD=MOD(NB,JAUNRL)
*
* COMPUTE HOUSEHOLDER INVERSE TRANSFORMATION FOR NGB BLOCK
*
*
* COPY ELEMENTARY REFLECTORS TO USEN
*
      IF((KBPIV .EQ. IPCOL) .AND. (IPCOL .EQ. IPROW))THEN
         DO J=1,NMOD
            DO I=1,NMOD
               USEN(J,IADDR+I)=A(IADDR+I,IADDR+J)
            ENDDO
         ENDDO
      END IF
*
* COPY ELEMENTARY REFLECTORS TO USEN
*
      CALL MPI_BCAST(USEN(1,IADDR+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *               KBPIV-1,MPI_COMM_ROW,IERR1)
      CALL MPI_BCAST(USEN(1,IADDR+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *               KBPIV-1,MPI_COMM_COL,IERR1)
      DO K=NMOD-2,1,-1
         JS=IS+K+1
         J1=JVE+1
         IF(JVBLK(IP+1) .GT. JS)THEN
            J1=1
         ELSE IF((JVBLK(IP+1) .LE. JS) .AND.
     *           (JS .LT. JVBLK(IP+2)))THEN
            J1=JS-JVBLK(IP+1)+1
         END IF
*
* INVERSE TRANSFORMATION
*
         DO J=J1,JVE
            CS=ZERO
            DO I=K+1,NMOD
               CS=CS+V(IS+I,J)*USEN(K,IADDR+I)
            ENDDO
            CS=CS*TAU(IS+K)
            DO I=K+1,NMOD
               V(IS+I,J)=V(IS+I,J)-CS*USEN(K,IADDR+I)
            ENDDO
         ENDDO
      ENDDO
      IF(NGB .EQ. 1)THEN
         GO TO 9999
      END IF
*
      KBPIV=MOD(NGB-2,NPROW)+1
      KTMP=(NGB-2)/NPROW+1
      IS=(KTMP-1)*NB
*
* COMPUTE HOUSEHOLDER INVERSE TRANSFORMATION FOR NGB-1 BLOCK
*
*
* RESTORE ELEMENTARY REFLECTORS TO UF, AND COPY TO USEN
*
      IF(IPCOL .EQ. KBPIV)THEN
         DO J=1,NB
            DO I=KTMP,NLB
               IG=(I-1)*NPROW*NB+(IPROW-1)*NB
               IL=(I-1)*NB
               DO K=1,NB
                  CA=A(IL+K,IS+J)
                  USEN(J,IL+K)=CA
                  UF(J  ,IG+K)=CA
               ENDDO
            ENDDO
         ENDDO
*
* RING COMM OF PIVOT BLOCK
* SEND    : IPROW
* RECEIVE : MOD(IPROW+NPROW-2,NPROW)
*
         ISEN =MOD(IPROW,NPROW)
         IREC =MOD(IPROW+NPROW-2,NPROW)
         ISIZE=(NLB-KTMP+1)*NB*NB
         DO K=1,NPROW-1
*
* SEND USEN, AND RECEIVE ELEMENTARY REFLECTORS
*
                  ITAG1=1
            CALL MPI_ISEND(USEN(1,IS+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *                     ISEN,ITAG1,MPI_COMM_ROW,REQ_S,IERR1)
            CALL MPI_RECV( UREC(1,IS+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *                     IREC,ITAG1,MPI_COMM_ROW,RWK,IERR2)
            CALL MPI_WAIT(REQ_S,SWK,IERR1)
            IADDR=MOD(IPROW+NPROW-K-1,NPROW)+1
*
* RESTORE ELEMENTARY REFLECTORS TO UF,AND COPY TO USEN
*
            DO J=1,NB
               DO I=KTMP,NLB
                  IG=(I-1)*NPROW*NB+(IADDR-1)*NB
                  IL=(I-1)*NB
                  DO L=1,NB
                     CA=UREC(J,IL+L)
                     USEN(J,IL+L)=CA
                     UF(J  ,IG+L)=CA
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      END IF
      ISIZE=(NLB-KTMP+1)*NB*NPROW*NB
*
* BROADCAST ELEMENTARY REFLECTORS UF TO ALL MPI PROCESSES
*
      CALL MPI_BCAST(UF(1,IS*NPROW+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *               KBPIV-1,MPI_COMM_COL,IERR1)
      IS=(NGB-2)*NB
*
* INVERSE TRANSFORMATION
*
      DO K=NB-KMOD,1,-1
         JS=IS+K+1
         J1=JVE+1
         IF(JVBLK(IP+1) .GT. JS)THEN
            J1=1
         ELSE IF((JVBLK(IP+1) .LE. JS) .AND.
     *           (JS .LT.JVBLK(IP+2)))THEN
            J1=JS-JVBLK(IP+1)+1
         END IF
         DO J=J1,JVE
            CS=ZERO
            DO I=IS+K+1,N
               CS=CS+UF(K,I)*V(I,J)
            ENDDO
            CS=CS*TAU(IS+K)
            DO I=IS+K+1,N
               V(I,J)=V(I,J)-CS*UF(K,I)
            ENDDO
         ENDDO
      ENDDO
*
* COMPUTE HOUSEHOLDER INVERSE TRANSFORMATION FOR NGB-2 TO 1 BLOCK
*
      DO KGB=NGB-2,1,-1
         KBPIV=MOD(KGB-1,NPROW)+1
         KTMP=(KGB-1)/NPROW+1
         IS=(KTMP-1)*NB
         IF(IPCOL .EQ. KBPIV)THEN
            DO J=1,NB
               DO I=KTMP,NLB
                  IG=(I-1)*NPROW*NB+(IPROW-1)*NB
                  IL=(I-1)*NB
                  DO K=1,NB
                     CA=A(IL+K,IS+J)
                     USEN(J,IL+K)=CA
                     UF(J  ,IG+K)=CA
                  ENDDO
               ENDDO
            ENDDO
*
* RING COMM OF PIVOT BLOCK
* SEND    : IPROW
* RECEIVE : MOD(IPROW+NPROW-2,NPROW)
*
            ISEN =MOD(IPROW,NPROW)
            IREC =MOD(IPROW+NPROW-2,NPROW)
            ISIZE=(NLB-KTMP+1)*NB*NB
            DO K=1,NPROW-1
*
* SEND USEN, AND RECEIVE ELEMENTARY REFLECTORS
*
                   ITAG1=1
               CALL MPI_ISEND(USEN(1,IS+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *                        ISEN,ITAG1,MPI_COMM_ROW,REQ_S,IERR1)
               CALL MPI_RECV( UREC(1,IS+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *                        IREC,ITAG1,MPI_COMM_ROW,RWK,IERR2)
               CALL MPI_WAIT(REQ_S,SWK,IERR1)
               IADDR=MOD(IPROW+NPROW-K-1,NPROW)+1
*
* RESTORE ELEMENTARY REFLECTORS TO UF,AND COPY TO USEN
*
               DO J=1,NB
                  DO I=KTMP,NLB
                     IG=(I-1)*NPROW*NB+(IADDR-1)*NB
                     IL=(I-1)*NB
                     DO L=1,NB
                        CA=UREC(J,IL+L)
                        USEN(J,IL+L)=CA
                        UF(J  ,IG+L)=CA
                     ENDDO
                  ENDDO
               ENDDO
            ENDDO
         END IF
         ISIZE=(NLB-KTMP+1)*NB*NPROW*NB
*
* BROADCAST ELEMENTARY REFLECTORS UF TO ALL MPI PROCESSES
*
         CALL MPI_BCAST(UF(1,IS*NPROW+1),ISIZE,MPI_DOUBLE_COMPLEX,
     *                  KBPIV-1,MPI_COMM_COL,IERR1)
*
* INVERSE TRANSFORMATION 4-2 UNROLL AND ROW-BLOCKING
*
         IS=(KGB-1)*NB
         DO K=NB-JAUNRL+1,1+NBMOD,-JAUNRL
            JS=IS+K+1
            J1=JVE+1
            IF(JVBLK(IP+1) .GT. JS)THEN
               J1=1
            ELSE IF((JVBLK(IP+1) .LE. JS) .AND.
     *              (JS .LT.JVBLK(IP+2)))THEN
               J1=JS-JVBLK(IP+1)+1
            END IF
*
* INVERSE TRANSFORMATION (CASE : MOD(NB,JUNRL) == 0)
*
*
* DOT PRODUCT CZ=<U*U^T>
*
            ZW12=UF(K  ,IS+K+2)*UF(K+1,IS+K+2)
     *          +UF(K  ,IS+K+3)*UF(K+1,IS+K+3)
            ZW13=UF(K  ,IS+K+3)*UF(K+2,IS+K+3)
            ZW23=UF(K+1,IS+K+3)*UF(K+2,IS+K+3)
            ZW14=ZERO
            ZW24=ZERO
            ZW34=ZERO
            DO I=IS+K+JAUNRL,N
               ZW12=ZW12+UF(K  ,I)*UF(K+1,I)
               ZW13=ZW13+UF(K  ,I)*UF(K+2,I)
               ZW23=ZW23+UF(K+1,I)*UF(K+2,I)
               ZW14=ZW14+UF(K  ,I)*UF(K+3,I)
               ZW24=ZW24+UF(K+1,I)*UF(K+3,I)
               ZW34=ZW34+UF(K+2,I)*UF(K+3,I)
            ENDDO
            ZC34=-TAU(IS+K+3)*ZW34
            ZC23=-TAU(IS+K+2)*ZW23
            ZC24=-TAU(IS+K+3)*ZW24+ZC23*ZC34
            ZC12=-TAU(IS+K+1)*ZW12
            ZC13=-TAU(IS+K+2)*ZW13+ZC12*ZC23
            ZC14=-TAU(IS+K+3)*ZW14-TAU(IS+K+2)*ZW13*ZC34
     *           -TAU(IS+K+3)*ZW24*ZC12+ZC12*ZC23*ZC34
*
* DOT PRODUCT CS=<U^T*V> (ROW-BLOCKING)
*
            JMOD=MOD(JVE-J1+1,JVUNRL)
            LB=(N-IS+K+JAUNRL)/IBLK+1
            DO J=J1,JVE-JMOD,JVUNRL
               WK(1,J  )=UF(K  ,IS+K+1)*V(IS+K+1,J  )
     *                  +UF(K  ,IS+K+2)*V(IS+K+2,J  )
     *                  +UF(K  ,IS+K+3)*V(IS+K+3,J  )
               WK(2,J  )=UF(K+1,IS+K+2)*V(IS+K+2,J  )
     *                  +UF(K+1,IS+K+3)*V(IS+K+3,J  )
               WK(3,J  )=UF(K+2,IS+K+3)*V(IS+K+3,J  )
               WK(4,J  )=ZERO
               WK(1,J+1)=UF(K  ,IS+K+1)*V(IS+K+1,J+1)
     *                  +UF(K  ,IS+K+2)*V(IS+K+2,J+1)
     *                  +UF(K  ,IS+K+3)*V(IS+K+3,J+1)
               WK(2,J+1)=UF(K+1,IS+K+2)*V(IS+K+2,J+1)
     *                  +UF(K+1,IS+K+3)*V(IS+K+3,J+1)
               WK(3,J+1)=UF(K+2,IS+K+3)*V(IS+K+3,J+1)
               WK(4,J+1)=ZERO
            ENDDO
            DO IB=1,LB
               ISTART=IS+K+JAUNRL-1+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+JAUNRL-1+IB*IBLK,N)
               DO J=J1,JVE-JMOD,JVUNRL
                  CS11=WK(1,J  )
                  CS12=WK(2,J  )
                  CS13=WK(3,J  )
                  CS14=WK(4,J  )
                  CS21=WK(1,J+1)
                  CS22=WK(2,J+1)
                  CS23=WK(3,J+1)
                  CS24=WK(4,J+1)
                  DO I=ISTART,IEND
                     CS11=CS11+UF(K  ,I)*V(I,J  )
                     CS12=CS12+UF(K+1,I)*V(I,J  )
                     CS13=CS13+UF(K+2,I)*V(I,J  )
                     CS14=CS14+UF(K+3,I)*V(I,J  )
                     CS21=CS21+UF(K  ,I)*V(I,J+1)
                     CS22=CS22+UF(K+1,I)*V(I,J+1)
                     CS23=CS23+UF(K+2,I)*V(I,J+1)
                     CS24=CS24+UF(K+3,I)*V(I,J+1)
                  ENDDO
                  WK(1,J  )=CS11
                  WK(2,J  )=CS12
                  WK(3,J  )=CS13
                  WK(4,J  )=CS14
                  WK(1,J+1)=CS21
                  WK(2,J+1)=CS22
                  WK(3,J+1)=CS23
                  WK(4,J+1)=CS24
               ENDDO
            ENDDO
            DO J=JVE-JMOD+1,JVE
               WK(1,J)=UF(K  ,IS+K+1)*V(IS+K+1,J)
     *                +UF(K  ,IS+K+2)*V(IS+K+2,J)
     *                +UF(K  ,IS+K+3)*V(IS+K+3,J)
               WK(2,J)=UF(K+1,IS+K+2)*V(IS+K+2,J)
     *                +UF(K+1,IS+K+3)*V(IS+K+3,J)
               WK(3,J)=UF(K+2,IS+K+3)*V(IS+K+3,J)
               WK(4,J)=ZERO
            ENDDO
            DO IB=1,LB
               ISTART=IS+K+JAUNRL-1+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+JAUNRL-1+IB*IBLK,N)
               DO J=JVE-JMOD+1,JVE
                  CS11=WK(1,J)
                  CS12=WK(2,J)
                  CS13=WK(3,J)
                  CS14=WK(4,J)
                  DO I=ISTART,IEND
                     CS11=CS11+UF(K  ,I)*V(I,J)
                     CS12=CS12+UF(K+1,I)*V(I,J)
                     CS13=CS13+UF(K+2,I)*V(I,J)
                     CS14=CS14+UF(K+3,I)*V(I,J)
                  ENDDO
                  WK(1,J)=CS11
                  WK(2,J)=CS12
                  WK(3,J)=CS13
                  WK(4,J)=CS14
               ENDDO
            ENDDO
*
* COMPUTE COEFFICIENT W=TAU*CZ*CS
*
            DO J=J1,JVE-JMOD,JVUNRL
               P14=WK(4,J  )
               P13=WK(3,J  )+ZC34*WK(4,J  )
               P12=WK(2,J  )+ZC23*WK(3,J  )+ZC24*WK(4,J  )
               P11=WK(1,J  )+ZC12*WK(2,J  )+ZC13*WK(3,J  )
     *                                     +ZC14*WK(4,J  )
               P24=WK(4,J+1)
               P23=WK(3,J+1)+ZC34*WK(4,J+1)
               P22=WK(2,J+1)+ZC23*WK(3,J+1)+ZC24*WK(4,J+1)
               P21=WK(1,J+1)+ZC12*WK(2,J+1)+ZC13*WK(3,J+1)
     *                                     +ZC14*WK(4,J+1)
               WK(1,J  )=TAU(IS+K  )*P11
               WK(2,J  )=TAU(IS+K+1)*P12
               WK(3,J  )=TAU(IS+K+2)*P13
               WK(4,J  )=TAU(IS+K+3)*P14
               WK(1,J+1)=TAU(IS+K  )*P21
               WK(2,J+1)=TAU(IS+K+1)*P22
               WK(3,J+1)=TAU(IS+K+2)*P23
               WK(4,J+1)=TAU(IS+K+3)*P24
            ENDDO
            DO J=JVE-JMOD+1,JVE
               P14=WK(4,J)
               P13=WK(3,J)+ZC34*WK(4,J)
               P12=WK(2,J)+ZC23*WK(3,J)+ZC24*WK(4,J)
               P11=WK(1,J)+ZC12*WK(2,J)+ZC13*WK(3,J)+ZC14*WK(4,J)
               WK(1,J)=TAU(IS+K  )*P11
               WK(2,J)=TAU(IS+K+1)*P12
               WK(3,J)=TAU(IS+K+2)*P13
               WK(4,J)=TAU(IS+K+3)*P14
            ENDDO
*
* COMPUTE AXPY V=V-W*U
*
            DO J=J1,JVE-JMOD,JVUNRL
               V(IS+K+1,J  )=V(IS+K+1,J  )-WK(1,J  )*UF(K  ,IS+K+1)
               V(IS+K+2,J  )=V(IS+K+2,J  )-WK(1,J  )*UF(K  ,IS+K+2)
     *                                    -WK(2,J  )*UF(K+1,IS+K+2)
               V(IS+K+3,J  )=V(IS+K+3,J  )-WK(1,J  )*UF(K  ,IS+K+3)
     *                                    -WK(2,J  )*UF(K+1,IS+K+3)
     *                                    -WK(3,J  )*UF(K+2,IS+K+3)
               V(IS+K+1,J+1)=V(IS+K+1,J+1)-WK(1,J+1)*UF(K  ,IS+K+1)
               V(IS+K+2,J+1)=V(IS+K+2,J+1)-WK(1,J+1)*UF(K  ,IS+K+2)
     *                                    -WK(2,J+1)*UF(K+1,IS+K+2)
               V(IS+K+3,J+1)=V(IS+K+3,J+1)-WK(1,J+1)*UF(K  ,IS+K+3)
     *                                    -WK(2,J+1)*UF(K+1,IS+K+3)
     *                                    -WK(3,J+1)*UF(K+2,IS+K+3)
            ENDDO
            DO IB=1,LB
               ISTART=IS+K+JAUNRL-1+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+JAUNRL-1+IB*IBLK,N)
               DO J=J1,JVE-JMOD,JVUNRL
                  DO I=ISTART,IEND
                     V(I,J  )=V(I,J  )-WK(1,J  )*UF(K  ,I)
     *                                -WK(2,J  )*UF(K+1,I)
     *                                -WK(3,J  )*UF(K+2,I)
     *                                -WK(4,J  )*UF(K+3,I)
                     V(I,J+1)=V(I,J+1)-WK(1,J+1)*UF(K  ,I)
     *                                -WK(2,J+1)*UF(K+1,I)
     *                                -WK(3,J+1)*UF(K+2,I)
     *                                -WK(4,J+1)*UF(K+3,I)
                  ENDDO
               ENDDO
            ENDDO
            DO J=JVE-JMOD+1,JVE
               V(IS+K+1,J)=V(IS+K+1,J)-WK(1,J)*UF(K  ,IS+K+1)
               V(IS+K+2,J)=V(IS+K+2,J)-WK(1,J)*UF(K  ,IS+K+2)
     *                                -WK(2,J)*UF(K+1,IS+K+2)
               V(IS+K+3,J)=V(IS+K+3,J)-WK(1,J)*UF(K  ,IS+K+3)
     *                                -WK(2,J)*UF(K+1,IS+K+3)
     *                                -WK(3,J)*UF(K+2,IS+K+3)
            ENDDO
            DO IB=1,LB
               ISTART=IS+K+JAUNRL-1+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+JAUNRL-1+IB*IBLK,N)
               DO J=JVE-JMOD+1,JVE
                  DO I=ISTART,IEND
                     V(I,J)=V(I,J)-WK(1,J)*UF(K  ,I)
     *                            -WK(2,J)*UF(K+1,I)
     *                            -WK(3,J)*UF(K+2,I)
     *                            -WK(4,J)*UF(K+3,I)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
*
* INVERSE TRANSFORMATION (CASE : MOD(NB,JUNRL) != 0)
*
         DO K=NBMOD,1,-1
            JS=IS+K+1
            J1=JVE+1
            IF(JVBLK(IP+1) .GT. JS)THEN
               J1=1
            ELSE IF((JVBLK(IP+1) .LE. JS) .AND.
     *              (JS .LT.JVBLK(IP+2)))THEN
               J1=JS-JVBLK(IP+1)+1
            END IF
*
* DOT PRODUCT CZ=<U*U^T>
*
            JMOD=MOD(JVE-J1+1,JVUNRL)
            LB=(N-IS+K)/IBLK+1
            DO J=J1,JVE
               WK(1,J)=ZERO
            ENDDO
*
* DOT PRODUCT CS=<U^T*V> (ROW-BLOCKING)
*
            DO IB=1,LB
               ISTART=IS+K+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+IB*IBLK,N)
               DO J=J1,JVE-JMOD,JVUNRL
                  CS11=ZERO
                  CS21=ZERO
                  DO I=ISTART,IEND
                     CS11=CS11+UF(K,I)*V(I,J  )
                     CS21=CS21+UF(K,I)*V(I,J+1)
                  ENDDO
                  WK(1,J  )=WK(1,J  )+CS11
                  WK(1,J+1)=WK(1,J+1)+CS21
               ENDDO
               DO J=JVE-JMOD+1,JVE
                  CS11=ZERO
                  DO I=ISTART,IEND
                     CS11=CS11+UF(K,I)*V(I,J)
                  ENDDO
                  WK(1,J)=WK(1,J)+CS11
               ENDDO
            ENDDO
*
* COMPUTE COEFFICIENT W=TAU*CZ*CS
*
            DO J=J1,JVE
               WK(1,J)=WK(1,J)*TAU(IS+K)
            ENDDO
*
* COMPUTE AXPY V=V-W*U
*
            DO IB=1,LB
               ISTART=IS+K+(IB-1)*IBLK+1
               IEND  =MIN(IS+K+IB*IBLK,N)
               DO J=J1,JVE-JMOD,JVUNRL
                  DO I=ISTART,IEND
                     V(I,J  )=V(I,J  )-WK(1,J  )*UF(K,I)
                     V(I,J+1)=V(I,J+1)-WK(1,J+1)*UF(K,I)
                  ENDDO
               ENDDO
               DO J=JVE-JMOD+1,JVE
                  DO I=ISTART,IEND
                     V(I,J)=V(I,J)-WK(1,J)*UF(K,I)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      ENDDO
*
 9999 CONTINUE
      RETURN
      END
