      SUBROUTINE J5MVLOWP(A,LDA,NB,JBLSTART,GAMMA,
     *                    K,NPROW,IPROW,IPCOL,NLB,KBPIV,KTMP,
     *                    P1,P2,Q1,Q2,PQ,QTPT,U,U1,U2,UL1,UL2,
     *                    C1,C2,CL1,CL2,LDIAG,
     *                    MPI_COMM_ROW,MPI_COMM_COL,MPI_COMM_DIG)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      INTEGER LDA,NB,NPROW
      COMPLEX*16 A(LDA,*)
      COMPLEX*16 P1(NLB*NB),P2(NLB*NB),Q1(NLB*NB),Q2(NLB*NB)
      COMPLEX*16 PQ(2*NB,NLB*NB),QTPT(2*NB,NLB*NB)
      COMPLEX*16 U(NLB*NB),UL1(NLB*NB),UL2(NLB*NB)
      COMPLEX*16 U1(NLB*NB),U2(NLB*NB)
      COMPLEX*16 CL1(NB),CL2(NB),C1(NB),C2(NB)
      COMPLEX*16 GAMMA
      INTEGER JBLSTART(NLB),LDIAG(NPROW)
      INTEGER IPROW,IPCOL,NLB,K,KBPIV,KTMP
      INTEGER MPI_COMM_ROW,MPI_COMM_COL,MPI_COMM_DIG
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5MVLOWP IS MATRIX-VECTOR PRODUCT.
*
*  ARGUMENTS
*  =========
*
*  A       (INPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  JBLSTART(INPUT) INTEGER ARRAY, DIMENSION (NLB).
*          STARTING ROW POSITION OF THE MATRIX A.
*
*  GAMMA   (INPUT) COMPLEX*16
*          THE K-TH PIVOT ELEMENTARY REFLECTORS.
*
*  K       (INPUT) INTEGER
*          THE COLUMN PIVOT POSITION IN THE PIVOT BLOCK.
*
*  NPROW   (INPUT) INTEGER
*          THE ROW SIZE OF PROCESS-MAP.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  KTMP    (INPUT) INTEGER
*          STARTING BLOCK OF THE MATRIX A.
*
*  P1      (INPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE K-TH PIVOT ELEMENTARY REFLECTORS.
*
*  P2      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  Q1      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  Q2      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  PQ      (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE COLUMN PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  QTPT    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE ROW PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  U       (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  U1      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  U2      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  UL1     (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  UL2     (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  C1      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB).
*
*  C2      (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB).
*
*  CL1     (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB).
*
*  CL2     (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NB).
*
*  LDIAG   (INPUT) INTEGER ARRAY, DIMENSION (NPROW).
*          DIAGONAL PROCESS LIST.
*
*  MPI_COMM(INPUT) INTEGER
*  _ROW    THE COMMUNICATOR OF ROW.
*
*  MPI_COMM(INPUT) INTEGER
*  _COL    THE COMMUNICATOR OF COLUMN.
*
*  MPI_COMM(INPUT) INTEGER
*  _DIG    THE COMMUNICATOR OF DIAGONAL.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 S,CS1,CS2,CA,CP
      INTEGER I,J,L,IS,IE,KS,KB
      INTEGER ISIZE,IDIAG,JDIAG,ILB,JLB,JTMP
      INTEGER IERR1,IERR2
      COMPLEX*16 ZERO
      PARAMETER (ZERO=DCMPLX(0.0D0,0.0D0))
*
      ILB=(KTMP-1)*NB+1
      IE=NLB*NB
      ISIZE=IE-ILB+1
      IDIAG=LDIAG(IPCOL)
      JDIAG=LDIAG(IPROW)
*
      JTMP=KTMP
      IF(IPCOL .LT. KBPIV)THEN
         JTMP=KTMP+1
      END IF
*
      JLB=KTMP
      IF(IPCOL .LE. KBPIV)THEN
         JLB=KTMP+1
      END IF
*
* BROADCAST P1, P2
*
      CALL MPI_BCAST(P1(ILB),ISIZE,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *               MPI_COMM_COL,IERR1)
      DO I=ILB,IE
         P2(I)=P1(I)
      ENDDO
      CALL MPI_BCAST(P2(ILB),ISIZE,MPI_DOUBLE_COMPLEX,IDIAG,
     *               MPI_COMM_ROW,IERR1)
*
* INITIALIZE MATRIX-VECTOR PARTIAL SUM ARRAY
*
      DO I=ILB,IE
         UL2(I)=ZERO
         U1(I) =ZERO
         U2(I) =ZERO
      ENDDO
*
* MATRIX-VECTOR PRODUCT A*P1 FOR DIAGONAL BLOCK
*
      IF(IPROW .EQ. IPCOL)THEN
         IS=(JTMP-1)*NB+1
         IE=NLB*NB
         KS=1
         IF(IPCOL .EQ. KBPIV)THEN
            IS=IS+K
            KS=KS+K
         END IF
*
         DO I=IS,IE
            UL2(I)=A(I,I)*P2(I)
         ENDDO
         DO L=JTMP,NLB
            IE=L*NB
            DO KB=KS,NB
               J =(L-1)*NB+KB
               IS=(L-1)*NB+KB+1
               CS1=ZERO
               CP=P2(J)
               DO I=IS,IE
                  CA=A(I,J)
                  CS1=CS1+CA*P1(I)
                  UL2(I)=UL2(I)+CA*CP
               END DO
               UL2(J)=UL2(J)+CS1
            ENDDO
            KS=1
         ENDDO
      END IF
*
* MATRIX-VECTOR PRODUCT A*P1, A*P2 FOR ALL BLOCK
*
      IE=NLB*NB
      DO L=JLB,NLB
         IS=(JBLSTART(L)-1)*NB+1
         DO KB=1,NB,2
            J=(L-1)*NB+KB
            CS1=ZERO
            CS2=ZERO
            DO I=IS,IE
               CS1=CS1+A(I,J  )*P1(I)
               CS2=CS2+A(I,J+1)*P1(I)
               UL2(I)=UL2(I)+A(I,J  )*P2(J  )
     *                      +A(I,J+1)*P2(J+1)
            ENDDO
            UL1(J  )=CS1
            UL1(J+1)=CS2
         ENDDO
      ENDDO
*
* MATRIX-VECTOR PRODUCT A*P1, A*P2 FOR PIVOT BLOCK
*
      IF(IPCOL .EQ. KBPIV)THEN
         IS=(JBLSTART(JTMP)-1)*NB+1
         DO KB=K+1,NB
            J=(KTMP-1)*NB+KB
            CS1=ZERO
            CP=P2(J)
            DO I=IS,IE
               CA=A(I,J)
               CS1=CS1+CA*P1(I)
               UL2(I)=UL2(I)+CA*CP
            ENDDO
            UL1(J)=CS1
         END DO
      END IF
*
* GATHER UL1, UL2
*
      CALL MPI_REDUCE(UL1(ILB),U1(ILB),ISIZE,MPI_DOUBLE_COMPLEX,
     *                MPI_SUM,IDIAG,MPI_COMM_ROW,IERR1)
      CALL MPI_REDUCE(UL2(ILB),U2(ILB),ISIZE,MPI_DOUBLE_COMPLEX,
     *                MPI_SUM,JDIAG,MPI_COMM_COL,IERR2)
      IF(IPCOL .EQ. IPROW)THEN
         IS=(JTMP-1)*NB+1
         IF(IPCOL .EQ. KBPIV)THEN
            IS=IS+K
         END IF
*
* SUM UP U1,U2 TO U
*
         DO I=IS,IE
            U(I)=GAMMA*(U1(I)+U2(I))
         ENDDO
*
* DOT PRODUCT OF THE CORRECTION VECTOR CL1, CL2
*
         DO J=1,K-1
            CS1=ZERO
            CS2=ZERO
            C1(J)=ZERO
            C2(J)=ZERO
            DO I=IS,IE
               CS1=CS1+QTPT(   J,I)*P1(I)
               CS2=CS2+QTPT(NB+J,I)*P1(I)
            ENDDO
            CL1(J)=CS1
            CL2(J)=CS2
         ENDDO
*
* SUM UP C1,C2 TO C1, C2
*
         IF(K .GT.1)THEN
            CALL MPI_REDUCE(CL1,C1,K-1,MPI_DOUBLE_COMPLEX,
     *                      MPI_SUM,KBPIV-1,MPI_COMM_DIG,IERR1)
            CALL MPI_REDUCE(CL2,C2,K-1,MPI_DOUBLE_COMPLEX,
     *                      MPI_SUM,KBPIV-1,MPI_COMM_DIG,IERR2)
*
            CALL MPI_BCAST(C1,K-1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *                     MPI_COMM_DIG,IERR1)
            CALL MPI_BCAST(C2,K-1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *                     MPI_COMM_DIG,IERR1)
         END IF
*
* U=U-C1*P-C2*Q
*
         DO J=1,K-1
            CS1=C1(J)
            CS2=C2(J)
            DO I=IS,IE
               U(I)=U(I)-GAMMA*(CS1*PQ(J,I)+CS2*PQ(NB+J,I))
            ENDDO
         ENDDO
*
* S=P^T*U*R/2
*
         CS1=ZERO
         DO I=IS,IE
            CS1=CS1+U(I)*P2(I)
         ENDDO
*
         CALL MPI_REDUCE(CS1,S,1,MPI_DOUBLE_COMPLEX,MPI_SUM,
     *                   KBPIV-1,MPI_COMM_DIG,IERR1)
*
         IF(IPCOL .EQ. KBPIV)THEN
            S=0.5D0*S*GAMMA
         END IF
         CALL MPI_BCAST(S,1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *                  MPI_COMM_DIG,IERR1)
*
* Q=U-S*P
*
         DO I=IS,IE
            Q1(I)=U(I)-S*P1(I)
            Q2(I)=Q1(I)
         ENDDO
      END IF
*
* BROADCAST Q1, Q2
*
      CALL MPI_BCAST(Q1(ILB),ISIZE,MPI_DOUBLE_COMPLEX,JDIAG,
     *               MPI_COMM_COL,IERR1)
      CALL MPI_BCAST(Q2(ILB),ISIZE,MPI_DOUBLE_COMPLEX,IDIAG,
     *               MPI_COMM_ROW,IERR1)
*
* P1, Q1 STORE TO PQ, AND P2, Q2 STORE TO QTPT
*
      DO I=ILB,IE
         PQ  (   K,I)=P1(I)
         PQ  (NB+K,I)=Q1(I)
         QTPT(   K,I)=Q2(I)
         QTPT(NB+K,I)=P2(I)
      ENDDO
*
      RETURN
      END
