      SUBROUTINE J5PAHOUS(A,LDA,NB,NLB,JBLSTART2,
     *                    K,NPROW,IPROW,IPCOL,KBLK,KBPIV,
     *                    DBUF,EBUF,TBUF,P1,PQ,QTPT,GAMMA,
     *                    MPI_COMM_ROW,MPI_COMM_COL)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      INTEGER LDA,NB,NLB
      COMPLEX*16 A(LDA,*)
      COMPLEX*16 PQ(2*NB,NLB*NB),QTPT(2*NB,NLB*NB)
      COMPLEX*16 P1(NLB*NB),DBUF(NLB*NB),EBUF(NLB*NB),TBUF(NLB*NB)
      COMPLEX*16 GAMMA
      INTEGER JBLSTART2(NLB)
      INTEGER KBLK,KBPIV,K,NPROW,IPROW,IPCOL
      INTEGER MPI_COMM_ROW,MPI_COMM_COL
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5PAHOUS IS PARTIAL HOUSEHOLDER TRANSFORMATION FOR PIVOT BLOCK.
*
*  ARGUMENTS
*  =========
*
*  A       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          ON ENTRY,
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*          ON EXIT,
*          THE ELEMENTARY REFLECTORS STORED K-TH COLUMN PIVOT.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*          
*  JBL     (INPUT) INTEGER ARRAY, DIMENSION (NLB).
*  START2  STARTING ROW POSITION OF THE MATRIX A.
*
*  K       (INPUT) INTEGER
*          THE COLUMN PIVOT POSITION IN THE PIVOT BLOCK.
*
*  NPROW   (INPUT) INTEGER
*          THE ROW SIZE OF PROCESS-MAP.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  KBLK    (INPUT) INTEGER
*          STARTING BLOCK OF THE MATRIX A.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  DBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  EBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  TBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
*  P1      (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE K-TH PIVOT ELEMENTARY REFLECTORS.
*
*  PQ      (INPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE COLUMN PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  QTPT    (INPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE ROW PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  GAMMA   (OUTPUT) COMPLEX*16
*          THE K-TH PIVOT ELEMENTARY REFLECTORS.
*
*  MPI_COMM(INPUT) INTEGER
*  _ROW    THE COMMUNICATOR OF ROW.
*
*  MPI_COMM(INPUT) INTEGER
*  _COL    THE COMMUNICATOR OF COLUMN.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 ALPHA,BETA,SIGMAL,SIGMA
      INTEGER I,J,IRED,IERR,ITAG
      INTEGER IADD,JB,JPIV,IB,IS,IE
      INTEGER SWK(MPI_STATUS_SIZE),RWK(MPI_STATUS_SIZE)
*
* PARAMETER
*
      COMPLEX*16 ZZERO,ZONE
      DOUBLE PRECISION RZERO
      PARAMETER (ZZERO=DCMPLX(0.0D0,0.0D0),ZONE=DCMPLX(1.0D0,0.0D0))
      PARAMETER (RZERO=0.0D0)
*
* SET END POINT OF MATRIX A
*
      IE=NLB*NB
*
* PARTIAL HOUSEHOLDER TRANSFORMATION FOR K-TH COLUMN
*
      IF(KBPIV .EQ. IPCOL)THEN
*
* PARTIAL RANK-2 MODIFICATION
*
         JB=KBLK*NB
         JPIV=JB+K
         IB=KBLK+1
         IF(IPCOL .EQ. IPROW) THEN
            IADD=1
            IS=(JBLSTART2(IB)-1)*NB+K
         ELSE
            IADD=0
            IS=(JBLSTART2(IB)-1)*NB+1
         END IF
         DO J=1,K-1
            DO I=IS,IE
               A(I,JPIV)=A(I,JPIV)-PQ(   J,I)*QTPT(   J,JPIV)
     *                            -PQ(NB+J,I)*QTPT(NB+J,JPIV)
            ENDDO
         ENDDO
*
* CALCULATE LOCAL SIGMA
*
         IS=IS+IADD
         SIGMAL=ZZERO
         DO I=IS,IE
            SIGMAL=SIGMAL+A(I,JPIV)*A(I,JPIV)
         ENDDO
*
* GATHER LOCAL SIGMA
*
         IF(K .NE. NB)THEN
            IRED=KBPIV-1
         ELSE
            IRED=MOD(KBPIV,NPROW)
         END IF
         SIGMA=ZZERO
         CALL MPI_REDUCE(SIGMAL,SIGMA,1,MPI_DOUBLE_COMPLEX,MPI_SUM,
     *                   IRED,MPI_COMM_ROW,IERR)
*
* CALCULATE ALPHA, BETA, GAMMA
*
         IF(IPROW-1 .EQ. IRED)THEN
            ALPHA=A(IS,JPIV)
            BETA =SQRT(SIGMA)
            IF(DBLE(ALPHA) .GE. RZERO)THEN
               BETA=-BETA
            END IF
            IF(BETA .NE. ZZERO)THEN
               GAMMA=ZONE/(SIGMA-ALPHA*BETA)
            ELSE
               GAMMA=ZZERO
               ALPHA=ZZERO
            END IF
            A(IS,JPIV)=ALPHA-BETA
         END IF
*
* SEND BETA TO DIAGONAL BLOCK
* (K AND NB ARE EQUAL)
*
              ITAG=1
         IF(KBPIV-1 .NE. IRED)THEN
            IF(IPROW-1 .EQ. IRED)THEN
            CALL MPI_SEND(BETA,1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *                    ITAG,MPI_COMM_ROW,SWK,IERR)
            ELSE IF(IPROW .EQ. IPCOL)THEN
            CALL MPI_RECV(BETA,1,MPI_DOUBLE_COMPLEX,IRED,
     *                    ITAG,MPI_COMM_ROW,RWK,IERR)
            END IF
         END IF
*
* BROADCAST THE GAMMA(ROW)
*
         CALL MPI_BCAST(GAMMA,1,MPI_DOUBLE_COMPLEX,IRED,
     *                  MPI_COMM_ROW,IERR)
*
* COPY ALPHA, BETA, GAMMA STORED AN ARRAY
*
         IF(IPROW .EQ. IPCOL)THEN
            DBUF(JPIV)=A(JPIV,JPIV)
            EBUF(JPIV)=BETA
            TBUF(JPIV)=GAMMA
         END IF
*
* COPY THE K-TH ELEMENTARY REFRECTOR
*
         DO I=IS,IE
            P1(I)=A(I,JPIV)
         ENDDO
      END IF
*
* BROADCAST THE GAMMA(COLUMN)
*
      CALL MPI_BCAST(GAMMA,1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *               MPI_COMM_COL,IERR)
*
      RETURN
      END
