      SUBROUTINE J5PARANK(A,LDA,NB,NLB,JBLSTART2,
     *                    K,NPROW,IPROW,IPCOL,KBLK,KBPIV,
     *                    DBUF,EBUF,PQ,QTPT,
     *                    MPI_COMM_ROW)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      INTEGER LDA,NB,NLB
      COMPLEX*16 A(LDA,*)
      COMPLEX*16 PQ(2*NB,NLB*NB),QTPT(2*NB,NLB*NB)
      COMPLEX*16 DBUF(NLB*NB),EBUF(NLB*NB)
      INTEGER JBLSTART2(NLB)
      INTEGER KBLK,KBPIV,K,NPROW,IPROW,IPCOL
      INTEGER MPI_COMM_ROW
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5PARANK IS PARTIAL RANK-2 MODIFICATION FOT NB-TH PIVOT.
*
*  ARGUMENTS
*  =========
*
*  A       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          ON ENTRY,
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*          ON EXIT,
*          RESULT OF PARTIAL RANK-2 MODIFICATION.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  JBL     (INPUT) INTEGER ARRAY, DIMENSION (NLB).
*  START2  STARTING ROW POSITION OF THE MATRIX A.
*
*  K       (INPUT) INTEGER
*          THE COLUMN PIVOT POSITION IN THE PIVOT BLOCK.
*
*  NPROW   (INPUT) INTEGER
*          THE ROW SIZE OF PROCESS-MAP.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  KBLK    (INPUT) INTEGER
*          STARTING BLOCK OF THE MATRIX A.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  DBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  EBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  PQ      (INPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE COLUMN PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  QTPT    (INPUT) COMPLEX*16 ARRAY, DIMENSION (2*NB,NLB*NB).
*          THE ROW PIVOT BLOCK ELEMENTARY REFLECTORS.
*
*  MPI_COMM(INPUT) INTEGER
*  _ROW    THE COMMUNICATOR OF ROW.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 BETA
      INTEGER I,J,IRED,IERR1,ITAG1
      INTEGER JB,JPIV,IB,IS,IE
      INTEGER SWK(MPI_STATUS_SIZE),RWK(MPI_STATUS_SIZE)
*
* PARAMETER
*
      COMPLEX*16 ZZERO,ZONE
      DOUBLE PRECISION RZERO
      PARAMETER (ZZERO=DCMPLX(0.0D0,0.0D0),ZONE=DCMPLX(1.0D0,0.0D0))
      PARAMETER (RZERO=0.0D0)
*
      IE=NLB*NB
*
      IF(KBPIV .EQ. IPCOL)THEN
         JB=KBLK*NB
         JPIV=JB+K
         IB=KBLK+1
         IF(IPCOL .EQ. IPROW) THEN
            IS=(JBLSTART2(IB)-1)*NB+K
         ELSE
            IS=(JBLSTART2(IB)-1)*NB+1
         END IF
*
* PARTIAL RANK-2 MODIFICATION FOR NB-TH PIVOT
*
         DO J=1,K-1
            DO I=IS,IE
               A(I,JPIV)=A(I,JPIV)-PQ(   J,I)*QTPT(   J,JPIV)
     *                            -PQ(NB+J,I)*QTPT(NB+J,JPIV)
            ENDDO
         ENDDO
         IRED=MOD(KBPIV,NPROW)
*
* SEND BETA TO DIAGONAL BLOCK
*
         ITAG1=1
         IF(IPROW-1 .EQ. IRED)THEN
            BETA=A(IS,JPIV)
            CALL MPI_SEND(BETA,1,MPI_DOUBLE_COMPLEX,KBPIV-1,
     *                    ITAG1,MPI_COMM_ROW,SWK,IERR1)
         ELSE IF(IPROW .EQ. IPCOL)THEN
            CALL MPI_RECV(BETA,1,MPI_DOUBLE_COMPLEX,IRED,
     *                    ITAG1,MPI_COMM_ROW,RWK,IERR1)
            DBUF(JPIV)=A(JPIV,JPIV)
            EBUF(JPIV)=BETA
         END IF
      END IF
*
      RETURN
      END
