      SUBROUTINE J5RNGCOM(N,A,LDA,D,E,TAU,NB,NLB,NPROW,NPCOL,
     *                    IPROW,IPCOL,KBPIV,NMOD,IDIAG,
     *                    DBUF,EBUF,TBUF,DSEN,ESEN,TSEN,MPI_COMM_DIG)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      INTEGER N,LDA
      COMPLEX*16 A(LDA,*)
      COMPLEX*16 D(N),E(N),TAU(N)
      COMPLEX*16 DBUF(NLB*NB),EBUF(NLB*NB),TBUF(NLB*NB)
      COMPLEX*16 DSEN(NLB*NB),ESEN(NLB*NB),TSEN(NLB*NB)
      INTEGER NB,NLB,NPROW,NPCOL,IPROW,IPCOL,KBPIV,NMOD,IDIAG
      INTEGER MPI_COMM_DIG
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5RNGCOM IS RESTORATION D, E, TAU BY DIAGONAL.
*
*  ARGUMENTS
*  =========
*
*  N       (INPUT) INTEGER
*          THE ORDER OF THE MATRIX.
*          N >= 2
*
*  A       (INPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  D       (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  E       (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  TAU     (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  NPROW   (INPUT) INTEGER
*          THE ROW SIZE OF PROCESS-MAP.
*
*  NPCOL   (INPUT) INTEGER
*          THE COLUMN SIZE OF PROCESS-MAP.
*
*  IPROW   (INPUT) INTEGER
*          THE ROW INDEX OF PROCESS-MAP.
*
*  IPCOL   (INPUT) INTEGER
*          THE COLUMN INDEX OF PROCESS-MAP.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  NMOD    (INPUT) INTEGER
*          THE NUMBER OF LAST BLOCK ELEMENTS.
*
*  IDIAG   (INPUT) INTEGER
*          INDEX OF DIAGONAL BLOCK.
*
*  DBUF    (INPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  EBUF    (INPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  TBUF    (INPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
*  DSEN    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  ESEN    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  TSEN    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*
*  MPI_COMM(INPUT) INTEGER
*  _DIG    THE COMMUNICATOR OF DIAGONAL.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      INTEGER I,J,K
      INTEGER IG,IL,NRES,IADDR
      INTEGER ISEN,IREC,ISIZE
*
      INTEGER IERR1,IERR2,ITAG1
      INTEGER REQ_S
      INTEGER SWK(MPI_STATUS_SIZE),RWK(MPI_STATUS_SIZE)
*
      IF(IPCOL .EQ. KBPIV)THEN
         IADDR=(NLB-1)*NB+NMOD-1
         DBUF(IADDR  )=A(IADDR  ,IADDR)
         DBUF(IADDR+1)=A(IADDR+1,IADDR+1)
         IF((NPROW .EQ. 1) .OR.
     *     ((NPROW .GT. 1) .AND. (NMOD .NE. 1)))THEN
            EBUF(IADDR  )=A(IADDR+1,IADDR)
         END IF
      END IF
*
* RESTORE DBUF TO D(AS WELL EBUF TO E, TBUF TO TAU)
*
      DO J=1,NLB-1
         IG=(J-1)*NPROW*NB+(IPROW-1)*NB
         IL=(J-1)*NB
         DO I=1,NB
            D  (IG+I)=DBUF(IL+I)
            E  (IG+I)=EBUF(IL+I)
            TAU(IG+I)=TBUF(IL+I)
         ENDDO
      ENDDO
      IF(IPCOL .EQ. KBPIV)THEN
         NRES=NMOD
      ELSE IF(IPCOL .GT. KBPIV)THEN
         NRES=0
      ELSE
         NRES=NB
      END IF
      IG=(NLB-1)*NPROW*NB+(IPROW-1)*NB
      IL=(NLB-1)*NB
      DO I=1,NRES
         D  (IG+I)=DBUF(IL+I)
         E  (IG+I)=EBUF(IL+I)
         TAU(IG+I)=TBUF(IL+I)
      ENDDO
*
* COPY DBUF TO DSEN(AS WELL, EBUF AND TBUF)
*
      DO I=1,NLB*NB
         DSEN(I)=DBUF(I)
         ESEN(I)=EBUF(I)
         TSEN(I)=TBUF(I)
      END DO
*
* RING COMM OF DIAGONAL BLOCK
* SEND    : IDIAG+1
* RECEIVE : MOD(IDIAG+NPROW-1,NPROW)
*
      ISEN =MOD(IDIAG+1,NPROW)
      IREC =MOD(IDIAG+NPROW-1,NPROW)
      ISIZE=NLB*NB
      DO K=1,NPCOL-1
*
* SEND DSEN, ESEN, TSEN AND RECEIVE DBUF, EBUF, TBUF
*
         ITAG1=1
         CALL MPI_ISEND(DSEN,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  ISEN,ITAG1,MPI_COMM_DIG,REQ_S,IERR1)
         CALL MPI_RECV( DBUF,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  IREC,ITAG1,MPI_COMM_DIG,RWK,IERR2)
         CALL MPI_WAIT(REQ_S,SWK,IERR1)
*
         CALL MPI_ISEND(ESEN,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  ISEN,ITAG1,MPI_COMM_DIG,REQ_S,IERR1)
         CALL MPI_RECV( EBUF,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  IREC,ITAG1,MPI_COMM_DIG,RWK,IERR2)
         CALL MPI_WAIT(REQ_S,SWK,IERR1)
*
         CALL MPI_ISEND(TSEN,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  ISEN,ITAG1,MPI_COMM_DIG,REQ_S,IERR1)
         CALL MPI_RECV( TBUF,ISIZE,MPI_DOUBLE_COMPLEX,
     *                  IREC,ITAG1,MPI_COMM_DIG,RWK,IERR2)
         CALL MPI_WAIT(REQ_S,SWK,IERR1)
*
         IADDR=MOD(IPROW+NPROW-K-1,NPROW)+1
*
* RESTORE DBUF TO D(AS WELL EBUF TO E, TBUF TO TAU)
*
         DO J=1,NLB-1
            IG=(J-1)*NPROW*NB+(IADDR-1)*NB
            IL=(J-1)*NB
            DO I=1,NB
               D  (IG+I)=DBUF(IL+I)
               E  (IG+I)=EBUF(IL+I)
               TAU(IG+I)=TBUF(IL+I)
            ENDDO
         ENDDO
         IF(IADDR .EQ. KBPIV)THEN
            NRES=NMOD
         ELSE IF(IADDR .GT. KBPIV)THEN
            NRES=0
         ELSE
            NRES=NB
         END IF
         IG=(NLB-1)*NPROW*NB+(IADDR-1)*NB
         IL=(NLB-1)*NB
         DO I=1,NRES
            D  (IG+I)=DBUF(IL+I)
            E  (IG+I)=EBUF(IL+I)
            TAU(IG+I)=TBUF(IL+I)
         ENDDO
*
* COPY DBUF TO DSEN(AS WELL, EBUF AND TBUF)
*
         DO I=1,NLB*NB
            DSEN(I)=DBUF(I)
            ESEN(I)=EBUF(I)
            TSEN(I)=TBUF(I)
         END DO
      ENDDO
*
      RETURN
      END
