      SUBROUTINE J5SERIAL(A,LDA,NB,NLB,KS,NMOD,DBUF,EBUF,TBUF)
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
* ARGUMENTS
*
      INTEGER NB,NLB,LDA,KS,NMOD
      COMPLEX*16 A(LDA,*),DBUF(NLB*NB),EBUF(NLB*NB),TBUF(NLB*NB)
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  J5SERIAL IS HOUSEHOLDER TRANSFORMATION FOR LAST BLOCK.
*
*  ARGUMENTS
*  =========
*
*  A       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (LDA,*)
*          ON ENTRY,
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*          ON EXIT,
*          THE ELEMENTARY REFLECTORS STORED LAST BLOCK.
*
*  LDA     (INPUT) INTEGER
*          LEADING DIMENSION OF A.
*
*  NB      (INPUT) INTEGER
*          BLOCK SIZE.
*
*  NLB     (INPUT) INTEGER
*          THE NUMBER OF BLOCKS IN MATRIX A.
*
*  KS      (INPUT) INTEGER
*          STARTING ROW POSITION OF THE MATRIX A.
*
*  NMOD    (INPUT) INTEGER
*          THE NUMBER OF LAST BLOCK ELEMENTS.
*
*  KBPIV   (INPUT) INTEGER
*          THE PIVOT BLOCK INDEX IN THE PROCESS-MAP.
*
*  DBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  EBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  TBUF    (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (NLB*NB).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 ALPHA,BETA,GAMMA,SIGMA,CS
      INTEGER I,J,K,KPIV
*
* PARAMETER
*
      COMPLEX*16 ZERO,ONE
      PARAMETER (ZERO=DCMPLX(0.0D0,0.0D0), ONE=DCMPLX(1.0D0,0.0D0))
*
      DO K=1,NMOD-2
         KPIV=KS+K
*
* CALCULATE SIGMA
*
         SIGMA=ZERO
         DO I=KPIV+1,KS+NMOD
            SIGMA=SIGMA+A(I,KPIV)*A(I,KPIV)
         ENDDO
*
* CALCULATE ALPHA, BETA, GAMMA
*
         BETA=SQRT(SIGMA)
         ALPHA=A(KPIV+1,KPIV)
         IF(DBLE(ALPHA) .GE. 0.0D0)THEN
            BETA=-BETA
         END IF
         IF(BETA .NE. ZERO)THEN
            GAMMA=ONE/(SIGMA-ALPHA*BETA)
         ELSE
            GAMMA=ZERO
            ALPHA=ZERO
         END IF
*
* COPY BETA STORED AN ARRAY
*
         EBUF(KPIV)=BETA
         A(KPIV+1,KPIV)=ALPHA-BETA
         DO J=K+1,NMOD
            CS=ZERO
*
* UPPER TRIANGULAR MATRIX-VECTOR
*
            DO I=K+1,J
               CS=CS+A(KS+J,KS+I)*A(KS+I,KPIV)
            ENDDO
*
* LOWER TRIANGULAR MATRIX-VECTOR
*
            DO I=J+1,NMOD
               CS=CS+A(KS+I,KS+J)*A(KS+I,KPIV)
            ENDDO
            TBUF(KS+J)=GAMMA*CS
         ENDDO
*
* DOT PRODUCT
*
         CS=ZERO
         DO I=K+1,NMOD
            CS=CS+TBUF(KS+I)*A(KS+I,KPIV)
         ENDDO
         ALPHA=-0.5D0*GAMMA*CS
*
* AXPY PRODUCT
*
         DO I=K+1,NMOD
            TBUF(KS+I)=TBUF(KS+I)+ALPHA*A(KS+I,KPIV)
         ENDDO
         DO J=K+1,NMOD
            DO I=J,NMOD
               A(KS+I,KS+J)=A   (KS+I,KS+J)
     *                     -A   (KS+I,KPIV)*TBUF(KS+J)
     *                     -TBUF(KS+I)     *A   (KS+J,KPIV)
            ENDDO
         ENDDO
*
* COPY ALPHA, GAMMA STORED AN ARRAY
*
         TBUF(KPIV)=GAMMA
         DBUF(KPIV)=A(KPIV,KPIV)
*
      ENDDO
*
      RETURN
      END
