      SUBROUTINE PZSYGHR(UPLO,N,A,IA,JA,DESCA,TAU,V,
     *                   WORK,LWORK,INFO,MAJOR)
*
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      CHARACTER*1 UPLO,MAJOR
      INTEGER N, IA,JA,LWORK,INFO
      COMPLEX*16 A(*),TAU(N),V(N,*),WORK(LWORK)
      INTEGER DESCA(9)
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  PZSYGHR COMPUTE HOUSEHOLDER INVERSE TRANSFORMATION.
*
*  ARGUMENTS
*  =========
*  UPLO    (INPUT) CHARACTER*1
*          THE COMPLEX SYMMETRIC MATRIX A IS STORED.
*          'L':  LOWER TRIANGULAR
*
*  N       (INPUT) INTEGER
*          THE ORDER OF THE MATRIX.
*          N >= 2
*
*  A       (INPUT) COMPLEX*16 ARRAY, DIMENSION (*).
*          THE PART OF A ELEMENTARY REFLECTORS.
*
*  IA      (INPUT) INTEGER
*          THE ROW INDEX IN THE GLOBAL ARRAY A INDICATING THE FIRST
*          ROW OF A.
*          IA = 1
*
*  JA      (INPUT) INTEGER
*          THE COLUMN INDEX IN THE GLOBAL ARRAY A INDICATING THE
*          FIRST COLUMN OF A.
*          JA = 1
*
*  DESCA   (INPUT) INTEGER ARRAY, DIMENSION (9).
*          THE ARRAY DESCRIPTOR FOR THE DISTRIBUTED MATRIX A.
*          DESCA(6) : BLOCK SIZE.
*                     DESCA(6) >= 8 AND MOD(DESCA(6),2) .EQ. 0
*          DESCA(9) : LEADING DIMENSION OF A.
*                     DESCA(9) >= NLB*NB
*                     (NB = DESCA(6), NLB = (N-1)/NB+1)
*
*  TAU     (INPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
*  V       (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N,*)
*          RESULT OF HOUSEHOLDER INVERSE TRANSFORMATION.
*
*  WORK    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (LWORK).
*
*  LWORK   (INPUT) INTEGER
*          THE DIMENSION OF THE ARRAY WORK.
*          LWORK >= NB*NPROW*NLB*NB + 2*NLB*NB*NB + 4*JVNUM
*          (NB = DESCA(6), NLB = (N-1)/NB+1,
*           NP = TOTAL NUMBER OF MPI PROCESS,
*           NPROW = LENGTH OF MPI PROCESS MAP,
*           JVNUM = (N-1)/NP+1)
*
*  INFO    (OUTPUT) INTEGER
*          =   0 : SUCCESSFUL EXIT.
*          <   0 : PARAMETER ERROR.
*          = 100 : INVALID NUMBER OF MPI PROCESS.
*
*  MAJOR   (INPUT) CHARACTER*1
*          PROCESS MAP TYPE.
*          'R': ROW MAJOR
*          'C': COLUMN MAJOR
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      INTEGER,ALLOCATABLE ::JVBLK(:)
      INTEGER NP,IP,NPROW,NPCOL,IPROW,IPCOL
      INTEGER NB,NGB,NLB,NMOD,LDA,JGB,JVS,JVE,JVNUM
      INTEGER I,J,JMOD,JT,IERR
      INTEGER IWP1,IWP2,IWP3,IWP4
*
      INTEGER MPI_COMM_ROW,MPI_COMM_COL
*
* INIT INFO
*
      INFO=0
*
* GET MPI SIZE AND RANK
*
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD,NP,IERR)
      CALL MPI_COMM_RANK(MPI_COMM_WORLD,IP,IERR)
*
* SET NPROW,NPCOL
*
      DO I=0,16
         NPROW=2**I
         IF(NPROW*NPROW .EQ. NP)THEN
            NPCOL=NPROW
            GO TO 1000
         END IF
      ENDDO
*
* MPI SIZE ERROR
*
      INFO=100
      GO TO 9999
 1000 CONTINUE
*
      NB=DESCA(6)
      LDA=DESCA(9)
      NGB=(N-1)/NB+1
      NLB=(NGB-1)/NPROW+1
      NMOD=MOD(N-1,NB)+1
      JVNUM=(N-1)/NP+1
* ======================================
* INPUT PARAMETER CHECK START
* ======================================
*
* CHECK UPLO
*
      IF(UPLO .NE. 'L')THEN
         INFO=-1
         GO TO 9999
      END IF
*
* CHECK N
*
      IF(N .LT. 2)THEN
         INFO=-2
         GO TO 9999
      END IF
*
* CHECK IA
*
      IF(IA .NE. 1)THEN
         INFO=-4
         GO TO 9999
      END IF
*
* CHECK JA
*
      IF(JA .NE. 1)THEN
         INFO=-5
         GO TO 9999
      END IF
*
* CHECK DESCA(6)
*
      IF((NB .LT. 8) .OR. (MOD(NB,2) .NE. 0))THEN
         INFO=-66
         GO TO 9999
      END IF
*
* CHECK DESCA(9)
*
      IF(LDA .LT. NLB*NB)THEN
         INFO=-69
         GO TO 9999
      END IF
*
* CHECK LWORK
*
      IF(LWORK .LT. NB*NPROW*NLB*NB+2*NLB*NB*NB+4*JVNUM)THEN
         INFO=-10
         GO TO 9999
      END IF
*
* CHECK MAJOR
*
      IF((MAJOR .NE. 'C') .AND. (MAJOR .NE. 'R'))THEN
         INFO=-12
         GO TO 9999
      END IF
* ======================================
* INPUT PARAMETER CHECK END
* ======================================
*
* SET IPROW, IPCOL
*
      IF(MAJOR .EQ. 'C')THEN
         IPROW=MOD(IP,NPROW)+1
         IPCOL=IP/NPROW+1
      ELSE IF(MAJOR .EQ. 'R')THEN
         IPROW=IP/NPCOL+1
         IPCOL=MOD(IP,NPCOL)+1
      END IF
*
* SET ROW COMUNICATOR
*
      CALL MPI_COMM_SPLIT(MPI_COMM_WORLD,IPCOL,IP,MPI_COMM_ROW,IERR)
*
* SET COLUMN COMUNICATOR
*
      CALL MPI_COMM_SPLIT(MPI_COMM_WORLD,IPROW,IP,MPI_COMM_COL,IERR)
*
* ALLOCATE JVBLK
*
      ALLOCATE(JVBLK(NP+1))
*
      JGB=N/NP
      JMOD=MOD(N,NP)
*
* SET JVBLK, JVS, JVE
*
      JVBLK(1)=1
      DO J=1,NP-1
         IF(J-1 .LT. JMOD)THEN
            JT=1
         ELSE
            JT=0
         END IF
         IF(JGB+JT .GT. 0)THEN
            JVBLK(J+1)=JVBLK(J)+JGB+JT
         ELSE
            JVBLK(J+1)=N+1
         END IF
      ENDDO
      JVBLK(NP+1)=N+1
      JVS=1
      JVE=JVBLK(IP+2)-JVBLK(IP+1)
*
* SET UNIT MATRIX
*
      CALL J5INIVEC(N,V,JVBLK,NP,IP,JVS,JVE)
*
* SET WORK POINTER
*
* IWP1:UF(NB*NPROW*NLB*NB)
* IWP2:USEN(NB*NLB*NB)
* IWP3:UREC(NB*NLB*NB)
* IWP4:WK(4*JVNUM)
*
      IWP1=1
      IWP2=IWP1+NB*NPROW*NLB*NB
      IWP3=IWP2+NLB*NB*NB
      IWP4=IWP3+NLB*NB*NB
*
* HOUSEHOLDER INVERSE TRANSFORMATION
*
      CALL J5MAKVEC(N,A,LDA,V,TAU,JVBLK,NP,IP,NB,NMOD,
     *              NPROW,NPCOL,IPROW,IPCOL,NLB,JVE,NGB,JVNUM,
     *              WORK(IWP1),WORK(IWP2),WORK(IWP3),WORK(IWP4),
     *              MPI_COMM_ROW,MPI_COMM_COL)
*
      DEALLOCATE(JVBLK)
 9999 CONTINUE
*
      RETURN
      END
