      SUBROUTINE PZSYTRD(UPLO,N,A,IA,JA,DESCA,D,E,TAU,
     *                   WORK,LWORK,INFO,MAJOR)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
*
      INCLUDE 'mpif.h'
*
* ARGUMENTS
*
      CHARACTER*1 UPLO,MAJOR
      INTEGER N,IA,JA,LWORK,INFO
      COMPLEX*16 A(*),D(N),E(N),TAU(N),WORK(LWORK)
      INTEGER DESCA(9)
*
* ----------------------------------------------------------------------
*  PURPOSE
*  =======
*
*  PZSYTRD REDUCE A COMPLEX SYMMETRIC MATRIX A TO COMPLEX SYMMETRIC
*  TRIDIAGONAL FORM T.
*
*  ARGUMENTS
*  =========
*  UPLO    (INPUT) CHARACTER*1
*          THE COMPLEX SYMMETRIC MATRIX A IS STORED.
*          'L':  LOWER TRIANGULAR
*
*  N       (INPUT) INTEGER
*          THE ORDER OF THE MATRIX.
*          N >= 2
*
*  A       (INPUT/OUTPUT) COMPLEX*16 ARRAY, DIMENSION (*).
*          ON ENTRY,
*          THE PART OF A COMPLEX SYMMETRIC MATRIX FOR TOTAL MATRIX.
*          ON EXIT,
*          THE PART OF A ELEMENTARY REFLECTORS.
*          
*  IA      (INPUT) INTEGER
*          THE ROW INDEX IN THE GLOBAL ARRAY A INDICATING THE FIRST
*          ROW OF A.
*          IA = 1
*
*  JA      (INPUT) INTEGER
*          THE COLUMN INDEX IN THE GLOBAL ARRAY A INDICATING THE
*          FIRST COLUMN OF A.
*          JA = 1
*
*  DESCA   (INPUT) INTEGER ARRAY, DIMENSION (9).
*          THE ARRAY DESCRIPTOR FOR THE DISTRIBUTED MATRIX A.
*          DESCA(6) : BLOCK SIZE.
*                     DESCA(6) >= 8 AND MOD(DESCA(6),2) .EQ. 0
*          DESCA(9) : LEADING DIMENSION OF A.
*                     DESCA(9) >= NLB*NB
*                     (NB = DESCA(6), NLB = (N-1)/NB+1)
*
*  D       (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  E       (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE SUB-DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T:
*
*  TAU     (OUTPUT) COMPLEX*16 ARRAY, DIMENSION (N).
*          THE ELEMENTARY REFLECTORS OF MATRIX A.
*
*  WORK    (WORKSPACE) COMPLEX*16 ARRAY, DIMENSION (LWORK).
*
*  LWORK   (INPUT) INTEGER
*          THE DIMENSION OF THE ARRAY WORK.
*          LWORK >= 12*NLB*NB + 4*NB + 4*NB*NLB*NB
*          (NB = DESCA(6), NLB = (N-1)/NB+1)
*
*  INFO    (OUTPUT) INTEGER
*          =   0 : SUCCESSFUL EXIT.
*          <   0 : PARAMETER ERROR.
*          = 100 : INVALID NUMBER OF MPI PROCESS.
*
*  MAJOR   (INPUT) CHARACTER*1
*          PROCESS MAP TYPE.
*          'R': ROW MAJOR
*          'C': COLUMN MAJOR
*
* ----------------------------------------------------------------------
*
* LOCAL VARIABLES
*
      COMPLEX*16 GAMMA
      INTEGER,ALLOCATABLE :: JBLSTART(:),JBLSTART2(:),LDIAG(:)
      INTEGER NP,NPROW,NPCOL,IP,IPROW,IPCOL,IDIAG,NB1
      INTEGER NB,NGB,NLB,KGB,LDA,KBPIV,KBLK,KTMP
      INTEGER I,K,IERR
      INTEGER NMOD,IADDR,IADDR2
      INTEGER IWP1,IWP2,IWP3,IWP4,IWP5,IWP6,IWP7,IWP8,IWP9,IWP10
      INTEGER IWP11,IWP12,IWP13,IWP14,IWP15,IWP16,IWP17,IWP18
*
* PARAMETER
*
      COMPLEX*16 ZERO
      PARAMETER (ZERO=DCMPLX(0.0D0,0.0D0))
*
      INTEGER MPI_COMM_ROW,MPI_COMM_COL,MPI_COMM_DIG
*
* INIT INFO
      INFO=0
*
* GET MPI SIZE AND RANK
*
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD,NP,IERR)
      CALL MPI_COMM_RANK(MPI_COMM_WORLD,IP,IERR)
*
* SET NPROW,NPCOL
*
      DO I=0,16
         NPROW=2**I
         IF(NPROW*NPROW .EQ. NP)THEN
            NPCOL=NPROW
            GO TO 1000
         END IF
      ENDDO
*
* MPI SIZE ERROR
*
      INFO=100
      GO TO 9999
 1000 CONTINUE
*
      NB  =DESCA(6)
      NGB =(N-1)/NB+1
      NLB =(NGB-1)/NPROW+1
      LDA =DESCA(9)
      NMOD=MOD(N-1,NB)+1
      NB1 =NB
* ======================================
* INPUT PARAMETER CHECK START
* ======================================
*
* CHECK UPLO
*
      IF(UPLO .NE. 'L')THEN
         INFO=-1
         GO TO 9999
      END IF
*
* CHECK N
*
      IF(N .LT. 2)THEN
         INFO=-2
         GO TO 9999
      END IF
*
* CHECK IA
*
      IF(IA .NE. 1)THEN
         INFO=-4
         GO TO 9999
      END IF
*
* CHECK JA
*
      IF(JA .NE. 1)THEN
         INFO=-5
         GO TO 9999
      END IF
*
* CHECK DESCA(6)
*
      IF((NB .LT. 8) .OR. (MOD(NB,2) .NE. 0))THEN
         INFO=-66
         GO TO 9999
      END IF
*
* CHECK DESCA(9)
*
      IF(LDA .LT. NLB*NB)THEN
         INFO=-69
         GO TO 9999
      END IF
*
* CHECK LWORK
*
      IF(LWORK .LT. 12*NLB*NB+4*NB+4*NB*NLB*NB)THEN
         INFO=-11
         GO TO 9999
      END IF
*
* CHECK MAJOR
*
      IF((MAJOR .NE. 'C') .AND. (MAJOR .NE. 'R'))THEN
         INFO=-13
         GO TO 9999
      END IF
* ======================================
* INPUT PARAMETER CHECK END
* ======================================
*
* SET IPROW, IPCOL
*
      IF(MAJOR .EQ. 'C')THEN
         IPROW=MOD(IP,NPROW)+1
         IPCOL=IP/NPROW+1
      ELSE IF(MAJOR .EQ. 'R')THEN
         IPROW=IP/NPCOL+1
         IPCOL=MOD(IP,NPCOL)+1
      END IF
*
* SET ROW COMUNICATOR
*
      CALL MPI_COMM_SPLIT(MPI_COMM_WORLD,IPCOL,IP,MPI_COMM_ROW,IERR)
*
* SET COLUMN COMUNICATOR
*
      CALL MPI_COMM_SPLIT(MPI_COMM_WORLD,IPROW,IP,MPI_COMM_COL,IERR)
*
* SET DIAGONAL COMUNICATOR
*
      IF(IPROW .EQ. IPCOL)THEN
         IDIAG=1
      ELSE
         IDIAG=0
      END IF
      CALL MPI_COMM_SPLIT(MPI_COMM_WORLD,IDIAG,IP,MPI_COMM_DIG,IERR)
*
* ALLOCATE JBLSTART, JBLSTART2, LDIAG
*
      ALLOCATE(JBLSTART(NLB),JBLSTART2(NLB),LDIAG(NPROW))
*
* SET JBLSTART, JBLSTART2
*
      IF(IPROW .LT. IPCOL)THEN
         IADDR2=1
      ELSE
         IADDR2=0
      END IF
      IF(IPROW .LE. IPCOL)THEN
         IADDR=1
      ELSE
         IADDR=0
      END IF
      DO I=1,NLB
         JBLSTART( I)=IADDR +I
         JBLSTART2(I)=IADDR2+I
      ENDDO
*
* SET DIAGONAL PROCESS LIST
*
      DO I=1,NPROW
         LDIAG(I)=I-1
      ENDDO
*
* INITIALIZE RESIDUAL BLOCK AND UPPER DIAGONAL ELEMENTS
*
      KBPIV=MOD(NGB-1,NPCOL)+1
      CALL J5INITZC(A,LDA,NB,NLB,IPROW,IPCOL,KBPIV,NMOD)
*
* SET WORK POINTER
*
* IWP1 :DBUF(NLB*NB)
* IWP2 :EBUF(NLB*NB)
* IWP3 :TBUF(NLB*NB)
* IWP4 :P1(NLB*NB)
* IWP5 :P2(NLB*NB)
* IWP6 :Q1(NLB*NB)
* IWP7 :Q2(NLB*NB)
* IWP8 :U(NLB*NB)
* IWP9 :U1(NLB*NB)
* IWP10:U2(NLB*NB)
* IWP11:UL1(NLB*NB)
* IWP12:UL2(NLB*NB)
* IWP13:C1(NB)
* IWP14:C2(NB)
* IWP15:CL1(NB)
* IWP16:CL2(NB)
* IWP17:PQ(2*NB*NLB*NB)
* IWP18:QTPT(2*NB*NLB*NB)
*
      IWP1 =1
      IWP2 =IWP1 +NLB*NB
      IWP3 =IWP2 +NLB*NB
      IWP4 =IWP3 +NLB*NB
      IWP5 =IWP4 +NLB*NB
      IWP6 =IWP5 +NLB*NB
      IWP7 =IWP6 +NLB*NB
      IWP8 =IWP7 +NLB*NB
      IWP9 =IWP8 +NLB*NB
      IWP10=IWP9 +NLB*NB
      IWP11=IWP10+NLB*NB
      IWP12=IWP11+NLB*NB
      IWP13=IWP12+NLB*NB
      IWP14=IWP13+NB
      IWP15=IWP14+NB
      IWP16=IWP15+NB
      IWP17=IWP16+NB
      IWP18=IWP17+2*NB*NLB*NB
* ======================================
* BLOCK HOUSEHOLDER TRANSFORMATION START
* ======================================
      DO KGB=1,NGB-1
         KBLK=(KGB-1)/NPROW
         KTMP=KBLK+1
         KBPIV=MOD(KGB-1,NPCOL)+1
         IF((KGB .EQ. NGB-1) .AND. (NMOD .EQ. 1))THEN
            NB1=NB-1
            DO I=(NLB-1)*NB+1,NLB*NB
               WORK(IWP17-1+(I-1)*2*NB+NB  )=ZERO
               WORK(IWP17-1+(I-1)*2*NB+2*NB)=ZERO
               WORK(IWP18-1+(I-1)*2*NB+NB  )=ZERO
               WORK(IWP18-1+(I-1)*2*NB+2*NB)=ZERO
            ENDDO
         END IF
         DO K=1,NB1
*
* PARTIAL HOUSEHOLDER TRANSFORMATION
*
            CALL J5PAHOUS(A,LDA,NB,NLB,JBLSTART2,
     *                    K,NPROW,IPROW,IPCOL,KBLK,KBPIV,
     *                    WORK(IWP1),WORK(IWP2),WORK(IWP3),WORK(IWP4),
     *                    WORK(IWP17),WORK(IWP18),GAMMA,
     *                    MPI_COMM_ROW,MPI_COMM_COL)
*
* MATRIX-VECTOR PRODUCT
*
      CALL J5MVLOWP(A,LDA,NB,JBLSTART,GAMMA,
     *                    K,NPROW,IPROW,IPCOL,NLB,KBPIV,KTMP,
     *                    WORK(IWP4),WORK(IWP5),WORK(IWP6),WORK(IWP7),
     *                    WORK(IWP17),WORK(IWP18),
     *                    WORK(IWP8),WORK(IWP9),WORK(IWP10),
     *                    WORK(IWP11),WORK(IWP12),
     *                    WORK(IWP13),WORK(IWP14),WORK(IWP15),
     *                    WORK(IWP16),LDIAG,
     *                    MPI_COMM_ROW,MPI_COMM_COL,MPI_COMM_DIG)
         ENDDO
*
* RANK-NB MODIFICATION
*
         CALL J5RANKNB(A,LDA,NB,NLB,JBLSTART2,IPCOL,KTMP,KBPIV,
     *                 WORK(IWP17),WORK(IWP18))
      ENDDO
* ======================================
* BLOCK HOUSEHOLDER TRANSFORMATION END
* ======================================
*
* SPECIAL CASE(NMOD .EQ. 1)
* RANK-2 MODIFICATION FOR NB-TH PIVOT
*
      IF(NB .NE. NB1)THEN
         KBLK=(NGB-2)/NPROW
         KBPIV=MOD(NGB-2,NPCOL)+1
         CALL J5PARANK(A,LDA,NB,NLB,JBLSTART2,NB,
     *                 NPROW,IPROW,IPCOL,KBLK,KBPIV,
     *                 WORK(IWP1),WORK(IWP2),
     *                 WORK(IWP17),WORK(IWP18),MPI_COMM_ROW)
      END IF
*
* LAST BLOCK HOUSEHOLDER TRANSFORMATION
*
      KBPIV=MOD(NGB-1,NPCOL)+1
      IF((IPROW .EQ. IPCOL) .AND. (IPCOL .EQ. KBPIV))THEN
         IADDR=(NLB-1)*NB
         CALL J5SERIAL(A,LDA,NB,NLB,IADDR,NMOD,
     *                 WORK(IWP1),WORK(IWP2),WORK(IWP3))
      END IF
*
* GATHER D, E, TAU
*
      IF(IPROW .EQ. IPCOL)THEN
         KBPIV=MOD(NGB-1,NPCOL)+1
         IDIAG=MOD(LDIAG(IPROW),NPROW)
         CALL J5RNGCOM(N,A,LDA,D,E,TAU,NB,NLB,NPROW,NPCOL,
     *                 IPROW,IPCOL,KBPIV,NMOD,IDIAG,
     *                 WORK(IWP1),WORK(IWP2),WORK(IWP3),
     *                 WORK(IWP4),WORK(IWP5),WORK(IWP6),
     *                 MPI_COMM_DIG)
      END IF
*
* BROADCAST D, E, TAU FROM DIAGONAL PROCESS
*
      CALL MPI_BCAST(D,N,MPI_DOUBLE_COMPLEX,LDIAG(IPROW),
     *               MPI_COMM_COL,IERR)
      CALL MPI_BCAST(E,N,MPI_DOUBLE_COMPLEX,LDIAG(IPROW),
     *               MPI_COMM_COL,IERR)
      CALL MPI_BCAST(TAU,N,MPI_DOUBLE_COMPLEX,LDIAG(IPROW),
     *               MPI_COMM_COL,IERR)
*
* DEALLOCATE JBLSTART, JBLSTART2, LDIAG
*
      DEALLOCATE(JBLSTART,JBLSTART2,LDIAG)
*
 9999 CONTINUE
*
      RETURN
      END
